# -*- coding: utf-8 -*-

from .memory_allocation_states import MemoryAllocationStatesEnum
from .memory_protections import MemoryProtectionsEnum
from .memory_types import MemoryTypesEnum
from .process_operations import ProcessOperationsEnum
from .standard_access_rights import StandardAccessRightsEnum
